package net.sourceforge.jFuzzyLogic.demo;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.AbstractAction;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.JSpinner;
import javax.swing.JToggleButton;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import net.sourceforge.jFuzzyLogic.rule.Variable;

/**
 *
 * @author pjl
 */
public abstract class FuzzyDemo {

	private boolean animateRules = false;
	private JPanel content;
	private JPanel controlPanel;
	protected int dilation = 6; // make time run slow
	protected File fclFile;
	protected FCLPanel fclPanel;
	JPanel fisPanel;
	protected boolean fuzzyControlActive = false;
	protected FuzzyController fuzzyController;
	private FuzzyDemoLayoutPanel fuzzyLayout;
	JMenuBar menuBar;
	protected Model model;
	private javax.swing.Timer timerG;
	private javax.swing.Timer timerM;
	private int timeStepG = 50; // millisec per frame
	private int timeStepM = 5; // millisec per controller update

	protected View view;

	/** Initializes the applet IPApplet */
	protected FuzzyDemo() {

		initComponents();

		timerG = new javax.swing.Timer(timeStepG, new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				//          System.out.println(" GUI update");
				view.modelStateChanged();
				if( animateRules ) fisPanel.repaint();
			}
		});

		timerG.setCoalesce(true);

		// model timestep stays constant but we vary the rate it is called to implement dilation  
		timerM = new javax.swing.Timer(timeStepM * dilation, new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				if( fuzzyControlActive && !view.isHumanActive() ) fuzzyController.control();
				//     System.out.println(" Model update");
				model.update(timeStepM / 1000.0);

			}
		});
		timerM.setCoalesce(false);

		fuzzyLayout.getTabPanel().addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				animateRules = fuzzyLayout.getTabPanel().getSelectedComponent() == fisPanel;

			}
		});

		//   content.getControlPanel().setBackground(Color.GREEN);
		//   content.getControlPanel().add(new JButton("LABEL"),BorderLayout.CENTER);

		controlPanel = new JPanel();
		controlPanel.setLayout(new BoxLayout(controlPanel, BoxLayout.X_AXIS));
		addControlsTo(controlPanel); // content.getControlPanel());
		content.add(controlPanel, BorderLayout.NORTH);
		content.validate();
	}

	protected abstract void addControlsImpl(JPanel panel);

	private void addControlsTo(JPanel cntrl) {

		addControlsImpl(cntrl);

		// add generic stuff
		cntrl.add(new JSeparator(SwingConstants.VERTICAL));
		JPanel sp = new JPanel();
		sp.add(new JLabel("Slow down by:"));
		final SpinnerNumberModel sm = new SpinnerNumberModel(dilation, 1, 50, 1);
		sm.addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				dilation = (sm).getNumber().intValue();
				timerM.setDelay(timeStepM * dilation);

			}
		});
		sp.add(new JSpinner(sm));
		cntrl.add(sp);

		cntrl.add(new JSeparator(SwingConstants.VERTICAL));

		addFuzzyControls(cntrl);
		cntrl.add(new JToggleButton(new AbstractAction("USE FUZZY CONTROL") {

			/**
			 * 
			 */
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent e) {
				fuzzyControlActive = ((JToggleButton) e.getSource()).isSelected();
			}
		}));

	}

	protected abstract void addFuzzyControls(JPanel cntrl);

	/**
	 *  top panel has all the buttons.
	 *  create generic and  delegate specific to subclasses.
	 * 
	 * @return
	 */
	protected JPanel createEditorPanel() {

		JPanel top = new JPanel();

		top.setLayout(new BorderLayout());
		try {
			top.add(fclPanel = new FCLPanel(fclFile), BorderLayout.CENTER);
		} catch(IOException ex) {
			Logger.getLogger(FuzzyDemoApplet.class.getName()).log(Level.SEVERE, null, ex);
		}

		JPanel cntrl = new JPanel();
		cntrl.setLayout(new BoxLayout(cntrl, BoxLayout.X_AXIS));

		// get subclass to add model spefic controls 

		cntrl.add(new JButton(new AbstractAction("Save") {

			/**
			 * 
			 */
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent e) {
				JFileChooser chooser = new JFileChooser();
				chooser.setDialogTitle("Save fuzzy rules");
				chooser.setFileFilter(new FCLFileFilter());
				if( fclFile != null ) chooser.setSelectedFile(fclFile);
				if( chooser.showSaveDialog(content) == JFileChooser.APPROVE_OPTION ) {
					File outFile = chooser.getSelectedFile();
					try {
						String str = fclPanel.getText();
						FileWriter out = new FileWriter(outFile);
						out.write(str);
						out.close();
					} catch(IOException ex) {
						Logger.getLogger(FuzzyDemoApplet.class.getName()).log(Level.SEVERE, null, ex);
					}
				}
			}
		}));

		cntrl.add(new JButton(new AbstractAction("Load") {

			/**
			 * 
			 */
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent e) {
				JFileChooser chooser = new JFileChooser();
				chooser.setDialogTitle("Open fuzzy rules");
				chooser.setFileFilter(new FCLFileFilter());
				if( fclFile != null ) chooser.setSelectedFile(fclFile);
				if( chooser.showOpenDialog(content) == JFileChooser.APPROVE_OPTION ) {
					File newFile = chooser.getSelectedFile();
					try {
						fuzzyController.reload(newFile);
						fclPanel.setFile(newFile);
						fclFile = newFile;
						initFisDebugPanel();
					} catch(IOException ex) {
						Logger.getLogger(FuzzyDemoApplet.class.getName()).log(Level.SEVERE, null, ex);
					}

				}
			}
		}));

		cntrl.add(new JButton(new AbstractAction("Apply") {

			/**
			 * 
			 */
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent e) {
				fuzzyController.reload(fclPanel.getText());
				initFisDebugPanel();
			}
		}));

		top.add(cntrl, BorderLayout.NORTH);
		return top;
	}

	public JPanel getControPanel() {
		return controlPanel;
	}

	public JMenuBar getJMenuBar() {
		return menuBar;
	}

	public Container getPanel() {
		return content;
	}

	private void initComponents() {

		menuBar = new JMenuBar();
		JMenu fileMenu = new JMenu("File");
		menuBar.add(fileMenu);

		fileMenu.add(new JMenuItem(new AbstractAction("exit") {

			/**
			 * 
			 */
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent e) {
				System.exit(0);
			}
		}));

		initModelImpl();
		content = new JPanel();
		content.setLayout(new BorderLayout());

		content.add(fuzzyLayout = new FuzzyDemoLayoutPanel((JPanel) view.getDisplayPanel()), BorderLayout.CENTER);
		fuzzyLayout.getTabPanel().add("Editor", createEditorPanel());

		initFisDebugPanel();
	}

	void initFisDebugPanel() {
		// Create a plot
		if( fisPanel != null ) {
			System.out.println(" Remove existing fis panel");
			fuzzyLayout.getTabPanel().remove(fisPanel);
		}
		List<Variable> list = fuzzyController.getVariables();
		fisPanel = new DemoPanelFis(fuzzyController.getVariables(), list.size(), 1);
		fuzzyLayout.getTabPanel().add("GRAPHS", fisPanel);
	}

	protected abstract void initModelImpl();

	public void start() {
		if( timerG.isRunning() ) timerG.restart();
		else timerG.start();
		if( timerM.isRunning() ) timerM.restart();
		else timerM.start();
	}

	public void stop() {
		timerG.stop();
		timerM.stop();
	}
}
